from django.template import Library
from django.conf import settings
from django.core.urlresolvers import reverse
from django.db.models import Sum

from emissions.models import Emission, Episode, NewsItem, SoundFile, Focus
from panikombo.models import Topik
from panikdb.stats.models import DailyStat
from combo.data.models import Page

register = Library()

@register.filter
def online_url(object):
    if isinstance(object, NewsItem):
        return settings.WEBSITE_BASE_URL + 'actus/' + object.slug
    if isinstance(object, Emission):
        return settings.WEBSITE_BASE_URL + 'emissions/' + object.slug + '/'
    if isinstance(object, Episode):
        return settings.WEBSITE_BASE_URL + 'emissions/' + object.emission.slug + '/' + object.slug + '/'
    if isinstance(object, SoundFile):
        return settings.WEBSITE_BASE_URL + 'emissions/' + object.episode.emission.slug + '/' + object.episode.slug
    if isinstance(object, Page):
        return settings.WEBSITE_BASE_URL + object.get_online_url().strip('/')
    return ''


@register.filter
def context_url(object):
    if isinstance(object, NewsItem) and object.emission:
        return reverse('emission-view', kwargs={'slug': object.emission.slug})
    if isinstance(object, Episode):
        return reverse('emission-view', kwargs={'slug': object.emission.slug})
    if isinstance(object, SoundFile):
        return reverse('emission-view', kwargs={'slug': object.episode.emission.slug})
    return None


@register.filter
def context_title(object):
    if isinstance(object, NewsItem) and object.emission:
        return object.emission.title
    if isinstance(object, Episode):
        return object.emission.title
    if isinstance(object, SoundFile):
        return object.episode.emission.title
    return None


@register.filter
def has_focus(object):
    if isinstance(object, NewsItem):
        object_type = 'newsitem'
    elif isinstance(object, Emission):
        object_type = 'emission'
    elif isinstance(object, Episode):
        object_type = 'episode'
    elif isinstance(object, SoundFile):
        object_type = 'soundfile'
    elif isinstance(object, Page):
        object_type = 'page'
    else:
        return False
    try:
        focus = Focus.objects.get(**{object_type: object})
    except Focus.DoesNotExist:
        return False
    return (focus.current == True)


@register.filter
def unset_focus_url(object):
    if isinstance(object, NewsItem):
        object_type = 'newsitem'
    elif isinstance(object, Emission):
        object_type = 'emission'
    elif isinstance(object, Episode):
        object_type = 'episode'
    elif isinstance(object, SoundFile):
        object_type = 'soundfile'
    elif isinstance(object, Page):
        object_type = 'page'
    return reverse('focus-unset', kwargs={
        'object_type': object_type,
        'object_id': object.id})


@register.filter
def set_focus_url(object):
    if isinstance(object, NewsItem):
        object_type = 'newsitem'
    elif isinstance(object, Emission):
        object_type = 'emission'
    elif isinstance(object, Episode):
        object_type = 'episode'
    elif isinstance(object, SoundFile):
        object_type = 'soundfile'
    elif isinstance(object, Page):
        object_type = 'page'
    return reverse('focus-set', kwargs={
        'object_type': object_type,
        'object_id': object.id})

@register.filter
def can_focus(object):
    if isinstance(object, NewsItem):
        return bool(object.category and object.image)
    elif isinstance(object, Emission):
        return True
    elif isinstance(object, Episode):
        return True
    elif isinstance(object, SoundFile):
        return bool(object.format)
    elif isinstance(object, Page):
        try:
            topik = Topik.objects.get(page=object)
        except Topik.DoesNotExist:
            return False
        return bool(topik.image)
    return False


@register.filter
def as_duration(value):
    if not value:
        return '-'
    return '%s:%02d' % (value//60, value%60)


@register.filter
def is_page(obj):
    return isinstance(obj, Page)

@register.filter
def has_topik(obj):
    return Topik.objects.filter(page=obj).count() == 1

@register.filter
def topik_id(obj):
    return Topik.objects.get(page=obj).id

@register.filter
def nb_visits(obj):
    t = DailyStat.objects.filter(soundfile_id=obj.id).aggregate(Sum('nb_visits'))
    return t['nb_visits__sum'] or '-'
